#ifndef LOGREADER_HPP
#define LOGREADER_HPP

#include "EntryType.hpp"
#include "ExtendedLog_global.h"
#include "LogEntry.hpp"

#include <QString>
#include <QVector>

namespace ExtendedLog
{
	/**
	 * @brief An abstract reader for extended log-files
	 */
	class EXTENDEDLOG_EXPORT LogReader
	{
		public:
			/**
			 * @brief The path of the log-file
			 */
			const QString filename;

			/**
			 * @brief Create a new log reader
			 * @param filename The file to use as log-file
			 *
			 * <b>Warning!</b> This does not automaticly read all entries from the log-file.
			 * This has to be done by each dreived class itself.
			 */
			LogReader(QString filename);

			/**
			 * @brief Get all entries from the log-file
			 * @return A vector of LogEntry objects
			 */
			QVector<LogEntry>& getAllEntries();

			/**
			 * @brief Get all entries with given type from the log-file
			 * @param type The EntryType to search for
			 * @return A filtered vector of LogEntry objects
			 */
			QVector<LogEntry>& getEntriesByType(EntryType type);

			/**
			 * @brief Get all entries beginning with the given message
			 * @param message The message to search for
			 * @return A filtered vector of LogEntry objects
			 *
			 * <b>Note:</b> The comparison is case-insensitve
			 */
			QVector<LogEntry>& getEntriesByMessage(QString message);

			/**
			 * @brief Get all entries with the given type and beginning with the given message
			 * @param type The EntryType to search for
			 * @param message The message to search for
			 * @return A filtered vector of LogEntry objects
			 *
			 * <b>Note:</b> The message-comparison is case-insensitive
			 */
			QVector<LogEntry>& getEntriesByMessgeAndType(EntryType type, QString message);
	
		protected:
			/**
			 * @brief A vector of all log entries; can be used for filtering
			 */
			QVector<LogEntry> allEntries;

			/**
			 * @brief Method stub for reading all entries
			 */
			virtual void readAllEntries() = 0;
	};
	
} // namespace ExtendedLog

#endif // LOGREADER_HPP
